# -*- coding: utf-8 -*-

"""
Enumerate all built-in ICON images.
"""

import os

dir_icons = os.path.join(os.path.dirname(os.path.abspath(__file__)), "icons")

class IconFileEnum:
    search = os.path.join(dir_icons, "search-96.png")
    refresh = os.path.join(dir_icons, "refresh-96.png")
    landline = os.path.join(dir_icons, "landline-96.png")
    pause = os.path.join(dir_icons, "pause-96.png")
    calendar = os.path.join(dir_icons, "calendar-96.png")
    start = os.path.join(dir_icons, "start-96.png")
    error = os.path.join(dir_icons, "error-96.png")
    reset = os.path.join(dir_icons, "reset-96.png")
    groupchat = os.path.join(dir_icons, "groupchat-96.png")
    question = os.path.join(dir_icons, "question-96.png")
    chat = os.path.join(dir_icons, "chat-96.png")
    plus = os.path.join(dir_icons, "plus-96.png")
    icons8 = os.path.join(dir_icons, "icons8-message-96.png")
    android = os.path.join(dir_icons, "android-96.png")
    minus = os.path.join(dir_icons, "minus-96.png")
    iphone = os.path.join(dir_icons, "iphone-96.png")
    check = os.path.join(dir_icons, "check-96.png")
    undo = os.path.join(dir_icons, "undo-96.png")
    stop = os.path.join(dir_icons, "stop-96.png")
    redo = os.path.join(dir_icons, "redo-96.png")
    close = os.path.join(dir_icons, "close-96.png")
    todo = os.path.join(dir_icons, "todo-96.png")
    remove = os.path.join(dir_icons, "remove-96.png")
